import ast
import logging
import six
import sys


TRUST_AST_TYPES = (ast.Call, ast.Module, ast.List, ast.Tuple, ast.Dict, ast.Name, ast.Num, ast.Str,
                   ast.Assign, ast.Load)

if sys.version_info[:2] == (3, 3):
    TRUST_AST_TYPES = TRUST_AST_TYPES + (ast.Bytes,)
elif six.PY3:
    TRUST_AST_TYPES = TRUST_AST_TYPES + (ast.Bytes, ast.NameConstant)


class InvalidETLConfig(Exception):
    pass


builtin_macros = [
    'KEEP_EVENT_',
    'DROP_EVENT_',
    'KEEP_FIELDS_',
    'DROP_FIELDS_',
    'RENAME_FIELDS_',
    'ALIAS_',
    'DISPATCH_EVENT_',
    'TRANSFORM_EVENT_',
    'KV_FIELDS_'
]

built_in_fns = ['V', 'JSON', 'CSV', 'REGEX', 'EMPTY', 'NO_EMPTY', 'DROP_F', 'KV', 'TSV', 'PSV', 'LOOKUP', 'SPLIT', 'ZIP']

built_in_ids = ['KV', 'ANY', 'ALL', 'F_TIME', 'F_META', 'F_TAGS', 'SPLIT', 'JSON', 'True', 'False', 'None']


logger = logging.getLogger(__name__)


class RestrictConfigParser(ast.NodeVisitor):
    def visit_ImportFrom(self, node):
        if node.module == 'aliyun.log.etl_core' and len(node.names) == 1 and node.names[0].name == '*':
            logger.info("[Passed] import detected: from aliyun.log.etl_core import *")
        else:
            raise InvalidETLConfig("unknown import: {0}".format(node.module))

    def visit_Call(self, node):
        if isinstance(node.func, ast.Name):
            if isinstance(node.func.ctx, ast.Load) and node.func.id in built_in_fns:
                logger.info("[Passed] known call detected")
            else:
                raise InvalidETLConfig("unknown call id detected: {0}".format(node.func.id))
        else:
            raise InvalidETLConfig("unknown call type detected: {0}".format(node.func))

    def visit_Name(self, node):
        if isinstance(node.ctx, ast.Store):
            for p in builtin_macros:
                if node.id.startswith(p):
                    logger.info('[Passed] assign detected: ', node.id)
                    break
            else:
                raise InvalidETLConfig('unknown assign detected: ', node.id)
        elif isinstance(node.ctx, ast.Load):
            if node.id in built_in_ids:
                logger.info(' [Passed] assigned name:', node.id)
            else:
                raise InvalidETLConfig('unknown load detected: ', node.id)
        else:
            raise InvalidETLConfig("unknown Name: {0}".format(node.id))

    def generic_visit(self, node):
        if isinstance(node, TRUST_AST_TYPES):
            logger.info("... known type detected: ", type(node))
        else:
            raise InvalidETLConfig("unknown type detected: {0}".format(type(node)))

        ast.NodeVisitor.generic_visit(self, node)

    def parse(self, code):
        self.visit(ast.parse(code))




from ..logexception import LogException


class SettingError(LogException):
    def __init__(self, ex=None, settings="", msg=""):
        if msg and settings:
            msg += u'\nInvalid Settings "{0}"'.format(settings)
        else:
            msg = msg or u'Invalid Settings "{0}"'.format(settings or 'unknown')

        super(SettingError, self).__init__('InvalidConfig', u'{0}\nDetail: {1}'.format(msg, ex))
        self.settings = settings




#!/usr/bin/env python
# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.

from .index_config import *
from .logresponse import LogResponse

__all__ = ['CreateIndexResponse', 'UpdateIndexResponse', 'DeleteIndexResponse', 'GetIndexResponse']


class CreateIndexResponse(LogResponse):
    """ The response of the create_index API from log.
    
    :type header: dict
    :param header: CreateIndexResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('CreateIndexResponse:')
        print('headers:', self.get_all_headers())


class UpdateIndexResponse(LogResponse):
    """ The response of the update_index API from log.
    
    :type header: dict
    :param header: UpdateIndexResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('UpdateIndexResponse:')
        print('headers:', self.get_all_headers())


class DeleteIndexResponse(LogResponse):
    """ The response of the delete_index API from log.
    
    :type header: dict
    :param header: DeleteIndexResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('DeleteIndexResponse:')
        print('headers:', self.get_all_headers())


class GetIndexResponse(LogResponse):
    """ The response of the get_index_config API from log.
    
    :type header: dict
    :param header: GetIndexResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.index_config = IndexConfig()
        self.index_config.from_json(resp)

    def get_index_config(self):
        """

        :return:
        """
        return self.index_config

    def log_print(self):
        """

        :return:
        """
        print('GetLogStoreResponse:')
        print('headers:', self.get_all_headers())
        print('index_configs:', str(self.index_config.to_json()))



#!/usr/bin/env python
# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.


from .. import IndexConfig, IndexKeyConfig, IndexLineConfig
from ..index_config import IndexJsonKeyConfig


class AliyunLogFieldType(object):
    TEXT = "text"
    LONG = "long"
    DOUBLE = "double"
    JSON = "json"


class DocRangeComparator(object):
    GT = "gt"
    LT = "lt"
    GTE = "gte"
    LTE = "lte"


class MappingIndexConverter(object):
    DEFAULT_TOKEN_LIST = [",", " ", "'", "\"", ";", "=", "(", ")", "[", "]", "{", "}", "?", "@", "&", "<", ">", "/",
                          ":", "\\n", "\\t", "\\r"]

    @classmethod
    def to_index_config(cls, mapping):
        if not mapping:
            return None
        line_config = IndexLineConfig(token_list=cls.DEFAULT_TOKEN_LIST, chinese=True)
        key_config_list = cls.to_key_config_list(properties=mapping["properties"])
        index_config = IndexConfig(line_config=line_config, key_config_list=key_config_list)
        return index_config

    @classmethod
    def to_key_config_list(cls, properties):
        key_config_list = {"_id": cls.handle_id()}
        if not properties:
            return key_config_list
        fields = set()
        for field_name, field_desc in properties.items():
            if field_name.lower() in fields:
                continue
            fields.add(field_name.lower())
            if "type" in field_desc:
                field_type = field_desc["type"]
                if field_type not in field_type_handlers:
                    continue
                key_config = field_type_handlers[field_type]()
                key_config_list[field_name] = key_config
            elif "properties" in field_desc:
                key_config = cls.handle_properties(field_desc["properties"])
                key_config_list[field_name] = key_config
            else:
                raise Exception("invalid field_desc '%s'" % field_desc)
        return key_config_list

    @classmethod
    def handle_id(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True
        )

    @classmethod
    def handle_text(cls):
        return IndexKeyConfig(
            token_list=cls.DEFAULT_TOKEN_LIST,
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
            chinese=True,
        )

    @classmethod
    def handle_keyword(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
            chinese=True,
        )

    @classmethod
    def handle_long(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.LONG,
            doc_value=True,
        )

    @classmethod
    def handle_integer(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.LONG,
            doc_value=True,
        )

    @classmethod
    def handle_short(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.LONG,
            doc_value=True,
        )

    @classmethod
    def handle_byte(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.LONG,
            doc_value=True,
        )

    @classmethod
    def handle_double(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.DOUBLE,
            doc_value=True,
        )

    @classmethod
    def handle_float(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.DOUBLE,
            doc_value=True,
        )

    @classmethod
    def handle_half_float(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.DOUBLE,
            doc_value=True,
        )

    @classmethod
    def handle_scaled_float(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.DOUBLE,
            doc_value=True,
        )

    @classmethod
    def handle_date(cls):
        return IndexKeyConfig(
            token_list=cls.DEFAULT_TOKEN_LIST,
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
        )

    @classmethod
    def handle_boolean(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
        )

    @classmethod
    def handle_integer_range(cls):
        json_key_config = IndexJsonKeyConfig()
        json_key_config.add_key(key_name=DocRangeComparator.GT, key_type=AliyunLogFieldType.LONG, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LT, key_type=AliyunLogFieldType.LONG, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.GTE, key_type=AliyunLogFieldType.LONG, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LTE, key_type=AliyunLogFieldType.LONG, doc_value=True)
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.JSON,
            json_key_config=json_key_config
        )

    @classmethod
    def handle_float_range(cls):
        json_key_config = IndexJsonKeyConfig()
        json_key_config.add_key(key_name=DocRangeComparator.GT, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LT, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.GTE, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LTE, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.JSON,
            json_key_config=json_key_config
        )

    @classmethod
    def handle_long_range(cls):
        json_key_config = IndexJsonKeyConfig()
        json_key_config.add_key(key_name=DocRangeComparator.GT, key_type=AliyunLogFieldType.LONG, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LT, key_type=AliyunLogFieldType.LONG, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.GTE, key_type=AliyunLogFieldType.LONG, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LTE, key_type=AliyunLogFieldType.LONG, doc_value=True)
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.JSON,
            json_key_config=json_key_config
        )

    @classmethod
    def handle_double_range(cls):
        json_key_config = IndexJsonKeyConfig()
        json_key_config.add_key(key_name=DocRangeComparator.GT, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LT, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.GTE, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LTE, key_type=AliyunLogFieldType.DOUBLE, doc_value=True)
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.JSON,
            json_key_config=json_key_config
        )

    @classmethod
    def handle_date_range(cls):
        json_key_config = IndexJsonKeyConfig()
        json_key_config.add_key(key_name=DocRangeComparator.GT, key_type=AliyunLogFieldType.TEXT, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LT, key_type=AliyunLogFieldType.TEXT, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.GTE, key_type=AliyunLogFieldType.TEXT, doc_value=True)
        json_key_config.add_key(key_name=DocRangeComparator.LTE, key_type=AliyunLogFieldType.TEXT, doc_value=True)
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.JSON,
            json_key_config=json_key_config
        )

    @classmethod
    def handle_ip_range(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
        )

    @classmethod
    def handle_geo_point(cls):
        return IndexKeyConfig(
            token_list=cls.DEFAULT_TOKEN_LIST,
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
        )

    @classmethod
    def handle_geo_shape(cls):
        return IndexKeyConfig(
            token_list=cls.DEFAULT_TOKEN_LIST,
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
        )

    @classmethod
    def handle_ip(cls):
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.TEXT,
            doc_value=True,
        )

    @classmethod
    def handle_properties(cls, properties):
        json_key_config = IndexJsonKeyConfig()
        key_configs = cls.parse_properties(None, properties)
        for key_name, key_config in key_configs.items():
            json_key_config.add_key(key_name=key_name, key_type=key_config.index_type, doc_value=key_config.doc_value)
            if len(json_key_config.json_keys) >= 50:
                break
        return IndexKeyConfig(
            index_type=AliyunLogFieldType.JSON,
            json_key_config=json_key_config
        )

    @classmethod
    def parse_properties(cls, prefix, properties):
        key_config_list = {}
        for field_name, field_desc in properties.items():
            if prefix:
                field_name = prefix + "." + field_name
            if "type" in field_desc:
                field_type = field_desc["type"]
                if field_type not in field_type_handlers:
                    continue
                key_config = field_type_handlers[field_type]()
                key_config_list[field_name] = key_config
            elif "properties" in field_desc:
                sub_key_config_list = cls.parse_properties(field_name, field_desc["properties"])
                key_config_list.update(sub_key_config_list)
            else:
                raise Exception("invalid field_desc '%s'" % field_desc)
        return key_config_list


field_type_handlers = {
    "text": MappingIndexConverter.handle_text,
    "keyword": MappingIndexConverter.handle_keyword,
    "long": MappingIndexConverter.handle_long,
    "integer": MappingIndexConverter.handle_integer,
    "short": MappingIndexConverter.handle_short,
    "byte": MappingIndexConverter.handle_byte,
    "double": MappingIndexConverter.handle_double,
    "float": MappingIndexConverter.handle_float,
    "half_float": MappingIndexConverter.handle_half_float,
    "scaled_float": MappingIndexConverter.handle_scaled_float,
    "date": MappingIndexConverter.handle_date,
    "boolean": MappingIndexConverter.handle_boolean,
    "integer_range": MappingIndexConverter.handle_integer_range,
    "float_range": MappingIndexConverter.handle_float_range,
    "long_range": MappingIndexConverter.handle_long_range,
    "double_range": MappingIndexConverter.handle_double_range,
    "date_range": MappingIndexConverter.handle_date_range,
    "ip_range": MappingIndexConverter.handle_ip_range,
    "geo_point": MappingIndexConverter.handle_geo_point,
    "geo_shape": MappingIndexConverter.handle_geo_shape,
    "ip": MappingIndexConverter.handle_ip,
}



# -*- coding: utf-8 -*-

from aliyun.log.logrequest import LogRequest
import json
import six

__all__ = ['CreateConsumerGroupRequest', 'ConsumerGroupGetCheckPointRequest',
           'ConsumerGroupHeartBeatRequest', 'ConsumerGroupUpdateCheckPointRequest']


class ConsumerGroupRequest(LogRequest):
    def __init__(self, project, logstore):
        LogRequest.__init__(self, project)
        # super(ConsumerGroupRequest, self).__init__(project)
        self.logstore = logstore

    def get_logstore(self):
        return self.logstore

    def set_logstore(self, logstore):
        self.logstore = logstore


class ConsumerGroupHeartBeatRequest(ConsumerGroupRequest):
    def __init__(self, project, logstore, consumer_group, consumer, shards):
        ConsumerGroupRequest.__init__(self, project, logstore)
        self.consumer_group = consumer_group
        self.consumer = consumer
        self.shards = shards

    def get_params(self):
        params = {
            'type': 'heartbeat',
            'consumer': self.consumer
        }
        return params

    def get_shards(self):
        return self.shards

    def set_shards(self, shards):
        self.shards = shards

    def get_request_body(self):
        return six.b(json.dumps(self.shards))


class ConsumerGroupGetCheckPointRequest(ConsumerGroupRequest):
    def __init__(self, project, logstore, consumer_group, shard):
        ConsumerGroupRequest.__init__(self, project, logstore)
        self.consumer_group = consumer_group
        self.shard = shard

    def get_params(self):
        if self.shard >= 0:
            return {'shard': self.shard}
        else:
            return {}

    def get_consumer_group(self):
        return self.consumer_group

    def set_consumer_group(self, consumer_group):
        self.consumer_group = consumer_group


class ConsumerGroupUpdateCheckPointRequest(ConsumerGroupRequest):
    def __init__(self, project, logstore, consumer_group, consumer, shard, check_point, force_success=True):
        ConsumerGroupRequest.__init__(self, project, logstore)
        self.consumer_group = consumer_group
        self.consumer = consumer
        self.shard = shard
        self.check_point = check_point
        self.force_success = force_success

    def get_consumer_group(self):
        return self.consumer_group

    def set_consumer_group(self, consumer_group):
        self.consumer_group = consumer_group

    def get_request_body(self):
        body_dict = {
            'shard': int(self.shard),
            'checkpoint': self.check_point
        }
        return six.b(json.dumps(body_dict))

    def get_request_params(self):
        params = {
            'type': 'checkpoint',
            'consumer': self.consumer,
            'forceSuccess': self.force_success
        }
        return params


class CreateConsumerGroupRequest(ConsumerGroupRequest):
    def __init__(self, project, logstore, consumer_group):
        ConsumerGroupRequest.__init__(self, project, logstore)
        # super(CreateConsumerGroupRequest, self).__init__(project, logstore)
        self.consumer_group = consumer_group

    def get_consumer_group(self):
        return self.consumer_group

    def set_consuemr_group(self, consumer_group):
        self.consumer_group = consumer_group



#!/usr/bin/env python
# encoding: utf-8

import sys
import base64
try:
    import collections.abc as collections_abc
except ImportError:
    import collections as collections_abc
import hashlib
import hmac
import socket
import six
from datetime import datetime, tzinfo, timedelta
from dateutil import parser
import re
import logging

logger = logging.getLogger(__name__)


def base64_encodestring(s):
    if six.PY2:
        return base64.encodestring(s)
    else:
        if isinstance(s, str):
            s = s.encode('utf8')
        return base64.encodebytes(s).decode('utf8')


def base64_decodestring(s):
    if six.PY2:
        return base64.decodestring(s)
    else:
        if isinstance(s, str):
            s = s.encode('utf8')
        return base64.decodebytes(s).decode('utf8')


class Util(object):
    @staticmethod
    def is_row_ip(ip):
        iparray = ip.split('.')
        if len(iparray) != 4:
            return False
        for tmp in iparray:
            if not tmp.isdigit() or int(tmp) >= 256:
                return False
        pattern = re.compile(r'^[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}$')
        if pattern.match(ip):
            return True
        return False

    @staticmethod
    def get_host_ip(logHost):
        """ If it is not match your local ip, you should fill the PutLogsRequest
        parameter source by yourself.
        """
        s = None
        try:
            s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
            s.connect((logHost, 80))
            ip = s.getsockname()[0]
            return ip
        except Exception:
            return '127.0.0.1'
        finally:
            if s:
                s.close()

    @staticmethod
    def compress_data(data):
        import zlib
        return zlib.compress(data, 6)

    @staticmethod
    def cal_md5(content):
        return hashlib.md5(content).hexdigest().upper()

    @staticmethod
    def hmac_sha1(content, key):
        if isinstance(content, six.text_type):  # hmac.new accept 8-bit str
            content = content.encode('utf-8')
        if isinstance(key, six.text_type):  # hmac.new accept 8-bit str
            key = key.encode('utf-8')

        hashed = hmac.new(key, content, hashlib.sha1).digest()
        return base64_encodestring(hashed).rstrip()

    @staticmethod
    def canonicalized_log_headers(headers):
        content = ''
        for key in sorted(six.iterkeys(headers)):
            if key[:6].lower() in ('x-log-', 'x-acs-'):  # x-log- header
                content += key + ':' + str(headers[key]) + "\n"
        return content

    @staticmethod
    def url_encode(params):
        for key, value in params.items():  # urllib.urlencode accept 8-bit str
            if isinstance(value, six.text_type):
                params[key] = value.encode('utf-8')
        return six.moves.urllib.parse.urlencode(params, True)

    @staticmethod
    def canonicalized_resource(resource, params):
        if params:
            urlString = ''
            for key, value in sorted(six.iteritems(params)):
                urlString += u"{0}={1}&".format(
                    key, value.decode('utf8') if isinstance(value, six.binary_type) else value)
            resource += '?' + urlString[:-1]
            if six.PY3:
                return resource
            else:
                return resource.encode('utf8')

        return resource

    @staticmethod
    def get_request_authorization(method, resource, key, params, headers):
        """ :return bytes (PY2) or string (PY2) """
        if not key:
            return six.b('')
        content = method + "\n"
        if 'Content-MD5' in headers:
            content += headers['Content-MD5']
        content += '\n'
        if 'Content-Type' in headers:
            content += headers['Content-Type']
        content += "\n"
        content += headers['Date'] + "\n"
        content += Util.canonicalized_log_headers(headers)
        content += Util.canonicalized_resource(resource, params)
        return Util.hmac_sha1(content, key)

    @staticmethod
    def to_ansi(data):
        pass

    @staticmethod
    def convert_unicode_to_str(data):
        """
        Py2, always translate to utf8 from unicode
        Py3, always translate to unicode
        :param data:
        :return:
        """
        if six.PY2 and isinstance(data, six.text_type):
            return data.encode('utf8')
        elif six.PY3 and isinstance(data, six.binary_type):
            return data.decode('utf8')
        elif isinstance(data, collections_abc.Mapping):
            return dict((Util.convert_unicode_to_str(k), Util.convert_unicode_to_str(v))
                        for k, v in six.iteritems(data))
        elif isinstance(data, collections_abc.Iterable) and not isinstance(data, (six.binary_type, six.string_types)):
            return type(data)(map(Util.convert_unicode_to_str, data))

        return data

    @staticmethod
    def h_v_t(header, key):
        """
        get header value with title
        try to get key from header and consider case sensitive
        e.g. header['x-log-abc'] or header['X-Log-Abc']
        :param header:
        :param key:
        :return:
        """
        if key not in header:
            key = key.title()

            if key not in header:
                raise ValueError("Unexpected header in response, missing: " + key + " headers:\n" + str(header))

        return header[key]

    @staticmethod
    def h_v_td(header, key, default):
        """
        get header value with title with default value
        try to get key from header and consider case sensitive
        e.g. header['x-log-abc'] or header['X-Log-Abc']
        :param header:
        :param key:
        :param default:
        :return:
        """
        if key not in header:
            key = key.title()

        return header.get(key, default)


ZERO = timedelta(0)


class UTC(tzinfo):
    """UTC"""

    def utcoffset(self, dt):
        return ZERO

    def tzname(self, dt):
        return "UTC"

    def dst(self, dt):
        return ZERO


utc = UTC()


def _get_total_seconds(delta):
    return ((delta.days * 86400 + delta.seconds) * 10**6 + delta.microseconds) / 10**6


def parse_timestamp(tm):
    if isinstance(tm, (int, float)) or \
            (isinstance(tm, (six.text_type, six.binary_type)) and tm.isdigit()):
        return int(tm)

    try:
        dt = parser.parse(tm)
    except ValueError as ex:
        try:
            # try to use dateparser to parse the format.
            from dateparser import parse
            try:
                dt = parse(tm)
                if dt is None:
                    raise ex
            except Exception as e:
                logger.error("fail to parse date: {0}, detail: {1}".format(tm, e))
                raise e

        except ImportError as ex2:
            raise ex

    if sys.version_info[:2] == (2, 6):
        if dt.tzinfo is None:
            return int(_get_total_seconds(dt - datetime(1970, 1, 1)))
        return int(_get_total_seconds(dt - datetime(1970, 1, 1, tzinfo=utc)))
    elif six.PY2:
        if dt.tzinfo is None:
            return int((dt - datetime(1970, 1, 1)).total_seconds())
        return int((dt - datetime(1970, 1, 1, tzinfo=utc)).total_seconds())
    else:
        return int(dt.timestamp())


def is_stats_query(query):
    """
    check if the query is a normal search or select query
    :param query:
    :return:
    """
    if not query:
        return False

    # remove all " enclosed strings
    nq = re.sub(r'"[^"]*"', '', query)

    # check if there's | .... select
    if re.findall(r'\|.*\bselect\b', nq, re.I|re.DOTALL):
        return True

    return False


class PrefixLoggerAdapter(logging.LoggerAdapter):
    def __init__(self, prefix, extra, *args, **kwargs):
        super(PrefixLoggerAdapter, self).__init__(*args, **kwargs)
        self._prefix = prefix
        self._extra = extra

    def process(self, msg, kwargs):
        kwargs['extra'] = kwargs.get('extra', {})
        kwargs['extra'].update(self._extra)

        return "{0}{1}".format(self._prefix, msg), kwargs



#!/usr/bin/env python
# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.


from ..logresponse import LogResponse


class MigrationResponse(LogResponse):
    def __init__(self, headers=None, resp=""):
        LogResponse.__init__(self, headers, resp)



#!/usr/bin/env python
# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.

from .logresponse import LogResponse


class PutLogsResponse(LogResponse):
    """ The response of the PutLogs API from log.
    
    :type header: dict
    :param header: PutLogsResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('PutLogsResponse:')
        print('headers:', self.get_all_headers())



#!/usr/bin/env python
# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.

__all__ = ['CreateMachineGroupResponse', 'DeleteMachineGroupResponse',
           'GetMachineGroupResponse', 'UpdateMachineGroupResponse',
           'ListMachineGroupResponse', 'ListMachinesResponse',
           'ApplyConfigToMachineGroupResponse', 'RemoveConfigToMachineGroupResponse',
           'GetMachineGroupAppliedConfigResponse', 'GetConfigAppliedMachineGroupsResponse']

from .util import Util
from .logresponse import LogResponse
from .machine_group_detail import MachineGroupDetail
from .machine_group_detail import MachineStatus


class CreateMachineGroupResponse(LogResponse):
    """ The response of the create_machine_group API from log.
    
    :type header: dict
    :param header: CreateMachineGroupResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('CreateMachineGroupResponse:')
        print('headers:', self.get_all_headers())


class DeleteMachineGroupResponse(LogResponse):
    """ The response of the delete_machine_group API from log.
    
    :type header: dict
    :param header: DeleteMachineGroupResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('DeleteMachineGroupResponse:')
        print('headers:', self.get_all_headers())


class GetMachineGroupResponse(LogResponse):
    """ The response of the get_machine_group API from log.
    
    :type header: dict
    :param header: GetMachineGroupResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.machine_group = MachineGroupDetail(None, None, None)
        self.machine_group.from_json(resp)

    def get_machine_group(self):
        """

        :return:
        """
        return self.machine_group

    def log_print(self):
        """

        :return:
        """
        print('GetMachineGroupResponse:')
        print('headers:', self.get_all_headers())
        print('machine_group', self.machine_group.to_json())


class UpdateMachineGroupResponse(LogResponse):
    """ The response of the update_machine_group API from log.
    
    :type header: dict
    :param header: UpdateMachineGroupResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('UpdateMachineGroupResponse:')
        print('headers:', self.get_all_headers())


class ListMachineGroupResponse(LogResponse):
    """ The response of the list_machine_group API from log.
    
    :type header: dict
    :param header: ListMachineGroupResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.count = int(resp["count"])
        self._total = int(resp["total"])
        self.machine_groups = Util.convert_unicode_to_str(resp.get("machinegroups", []))

    def get_machine_group(self):
        """

        :return:
        """
        return self.machine_groups

    def get_machine_group_count(self):
        """

        :return:
        """
        return self.count

    def get_machine_group_total(self):
        """

        :return:
        """
        return self._total

    def get_count(self):
        return self.count

    def get_total(self):
        return self._total

    @property
    def total(self):
        return self._total

    def log_print(self):
        """

        :return:
        """
        print('ListMachineGroupResponse:')
        print('headers:', self.get_all_headers())
        print('count:', str(self.count))
        print('total:', str(self._total))
        print('machine_groups:', str(self.machine_groups))

    def merge(self, response):
        if not isinstance(response, ListMachineGroupResponse):
            raise ValueError("passed response is not a ListMachineGroupResponse: " + str(type(response)))

        self.count += response.get_count()
        self._total = response.get_total()  # use the latest total count
        self.machine_groups.extend(response.get_machine_group())

        # update body
        self.body = {
            'count': self.count,
            'total': self._total,
            'machinegroups': self.machine_groups
        }

        return self


class ListMachinesResponse(LogResponse):
    """ The response of the list_machines API from log.
    
    :type header: dict
    :param header: ListMachinesResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.count = resp['count']
        self.total = resp['total']
        self.machines = []
        for machine_status in resp['machines']:
            machine = MachineStatus(machine_status['ip'], machine_status['machine-uniqueid'],
                                    machine_status['userdefined-id'], machine_status['lastHeartbeatTime'])
            self.machines.append(machine)

    def get_machine_count(self):
        """

        :return:
        """
        return self.count

    def get_machine_total(self):
        """

        :return:
        """
        return self.total

    def get_machines(self):
        """

        :return:
        """
        return self.machines

    def get_count(self):
        return self.count

    def get_total(self):
        return self.total

    def log_print(self):
        print('GetMachineGroupResponse:')
        print('headers:', self.get_all_headers())
        print('machine_count', self.count)
        print('machine_total', self.total)
        print('machines:', self.machines)

    def merge(self, response):
        if not isinstance(response, ListMachinesResponse):
            raise ValueError("passed response is not a ListMachineGroupResponse: " + str(type(response)))

        self.count += response.get_count()
        self.total = response.get_total()  # use the latest total count
        self.machines.extend(response.get_machines())

        # update body
        self.body['count'] = self.count
        self.body['total'] = self.total
        self.body['machines'].extend(response.get_body()['machines'])

        return self


class ApplyConfigToMachineGroupResponse(LogResponse):
    """ The response of the apply_config_to_machine_group API from log.
    
    :type header: dict
    :param header: ApplyConfigToMachineGroupResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('ApplyConfigToMachineGroupResponse:')
        print('headers:', self.get_all_headers())


class RemoveConfigToMachineGroupResponse(LogResponse):
    """ The response of the remove_config_to_machine_group API from log.
    
    :type header: dict
    :param header: RemoveConfigToMachineGroupResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('RemoveConfigToMachineGroupResponse:')
        print('headers:', self.get_all_headers())


class GetMachineGroupAppliedConfigResponse(LogResponse):
    """ The response of the get_machine_group_applied_config API from log.
    
    :type header: dict
    :param header: GetMachineGroupAppliedConfigResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.count = resp['count']
        self.configs = resp['configs']

    def get_config_count(self):
        """

        :return:
        """
        return self.count

    def get_configs(self):
        """

        :return:
        """
        return self.configs

    def log_print(self):
        print('GetMachineGroupAppliedConfigResponse:')
        print('headers:', self.get_all_headers())
        print('count:', self.count)
        print('configs:', self.configs)


class GetConfigAppliedMachineGroupsResponse(LogResponse):
    """ The response of the get_config_applied_machine_group API from log.
    
    :type header: dict
    :param header: GetConfigAppliedMachineGroupsResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.count = resp['count']
        self.machine_groups = resp['machinegroups']

    def get_machine_group_count(self):
        """

        :return:
        """
        return self.count

    def get_machine_groups(self):
        """

        :return:
        """
        return self.machine_groups

    def log_print(self):
        """

        :return:
        """
        print('GetConfigAppliedMachineGroupsResponse:')
        print('headers:', self.get_all_headers())
        print('count:', self.count)
        print('machine_groups:', self.machine_groups)



#!/usr/bin/env python
# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.


class OdpsShipperConfig(object):
    """ Odps shipper config

    :type odps_endpoint: string
    :param odps_endpoint: the odps endpoint

    :type odps_project: string
    :param odps_project: the odps project name

    :type odps_table: string
    :param odps_table: the odps table name

    :type log_fields_list: string array
    :param log_fields_list: the log field(keys in log) list mapping to the odps table column. e.g log_fields_list=['__time__', 'key_a', 'key_b'], the $log_time, $log_key_a, $log_key_b will mapping to odps table column No.1, No.2, No.3

    :type partition_column: string array
    :param partition_column: the log fields mapping to odps table partition column

    :type partition_time_format: string
    :param partition_time_format: the time format of __partition_time__, e.g yyyy_MM_dd_HH_mm

    """

    def __init__(self, odps_endpoint, odps_project, odps_table, log_fields_list, partition_column,
                 partition_time_format, bufferInterval=1800):
        self.odps_endpoint = odps_endpoint
        self.odps_project = odps_project
        self.odps_table = odps_table
        self.log_fields_list = log_fields_list
        self.partition_column = partition_column
        self.partition_time_format = partition_time_format
        self.buffer_interval = bufferInterval

    def to_json(self):
        json_value = {"odpsEndpoint": self.odps_endpoint, "odpsProject": self.odps_project,
                      "odpsTable": self.odps_table, "fields": self.log_fields_list,
                      "partitionColumn": self.partition_column, "partitionTimeFormat": self.partition_time_format,
                      "bufferInterval": self.buffer_interval}
        return json_value


class OssShipperConfig(object):
    """A oss ship config

    :type oss_bucket: string
    :param oss_bucket: the oss bucket name

    :type oss_prefix: string
    :param oss_prefix: the the prefix path where to save the log

    :type oss_role_arn: string
    :param oss_role_arn: the ram arn used to get the temporary write permission to the oss bucket

    :type buffer_interval: int
    :param buffer_interval: the time(seconds) to buffer before save to oss

    :type buffer_mb: int
    :param buffer_mb: the data size(MB) to buffer before save to oss

    :type compress_type: string
    :param compress_type: the compress type, only support 'snappy' or 'none'
    """

    def __init__(self, oss_bucket, oss_prefix, oss_role_arn, buffer_interval=300, buffer_mb=128,
                 compress_type='snappy'):
        self.oss_bucket = oss_bucket
        self.oss_prefix = oss_prefix
        self.oss_role_arn = oss_role_arn
        self.buffer_interval = buffer_interval
        self.buffer_mb = buffer_mb
        self.compress_type = compress_type

    def to_json(self):
        json_value = {'ossBucket': self.oss_bucket, 'ossPrefix': self.oss_prefix, 'roleArn': self.oss_role_arn,
                      'bufferInterval': self.buffer_interval, 'bufferSize': self.buffer_mb,
                      'compressType': self.compress_type}
        print(json_value)
        return json_value


class ShipperTask(object):
    """A shipper task

    :type task_id: string
    :param task_id: the task id

    :type task_status: string
    :param task_status: one of ['success', 'running', 'fail']

    :type task_message: string
    :param task_message: the error message of task_status is 'fail'

    :type task_create_time:  int
    :param task_create_time: the task create time (timestamp from 1970.1.1)

    :type task_last_data_receive_time: int
    :param task_last_data_receive_time: last log data receive time (timestamp)

    :type task_finish_time: int
    :param task_finish_time: the task finish time (timestamp)
    """

    def __init__(self, task_id, task_status, task_message, task_create_time, task_last_data_receive_time,
                 task_finish_time):
        self.task_id = task_id
        self.task_status = task_status
        self.task_message = task_message
        self.task_create_time = task_create_time
        self.task_last_data_receive_time = task_last_data_receive_time
        self.task_finish_time = task_finish_time

    def to_json(self):
        json_value = {'id': self.task_id, 'taskStatus': self.task_status, 'taskMessage': self.task_message,
                      'taskCreateTime': self.task_create_time,
                      'taskLastDataReceiveTime': self.task_last_data_receive_time,
                      'taskFinishTime': self.task_finish_time}
        return json_value





class CheckPointException(Exception):
    def __init__(self, message, cause=None):
        if cause is not None:
            super(CheckPointException, self).__init__(message, cause)
        else:
            super(CheckPointException, self).__init__(message)


class ClientWorkerException(Exception):

    def __init__(self, message, cause=None):
        if cause is not None:
            super(ClientWorkerException, self).__init__(message, cause)
        else:
            super(ClientWorkerException, self).__init__(message)



# encoding: utf-8

# Copyright (C) Alibaba Cloud Computing
# All rights reserved.

__all__ = ['CreateExternalStoreResponse', 'DeleteExternalStoreResponse', 'GetExternalStoreResponse',
           'UpdateExternalStoreResponse', 'ListExternalStoreResponse']

from .util import Util
from .logresponse import LogResponse
from .external_store_config import ExternalStoreConfigBase


class CreateExternalStoreResponse(LogResponse):
    """ The response of the create_logstore API from log.
    
    :type header: dict
    :param header: CreateExternalStoreResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('CreateExternalStoreResponse:')
        print('headers:', self.get_all_headers())


class DeleteExternalStoreResponse(LogResponse):
    """ The response of the delete_logstore API from log.
    
    :type header: dict
    :param header: DeleteExternalStoreResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('DeleteExternalStoreResponse:')
        print('headers:', self.get_all_headers())


class GetExternalStoreResponse(LogResponse):
    """ The response of the get_logstore API from log.
    
    :type header: dict
    :param header: GetExternalStoreResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.externalStoreConfig = ExternalStoreConfigBase.from_json(resp)

    def get_external_store_config(self):
        """

        :return:
        """
        return self.externalStoreConfig

    def log_print(self):
        """

        :return:
        """
        print('GetExternalStoreResponse:')
        print('headers:', self.get_all_headers())
        self.externalStoreConfig.log_print()


class UpdateExternalStoreResponse(LogResponse):
    """ The response of the update_logstore API from log.
    
    :type header: dict
    :param header: UpdateExternalStoreResponse HTTP response header
    """

    def __init__(self, header, resp=''):
        LogResponse.__init__(self, header, resp)

    def log_print(self):
        print('UpdateExternalStoreResponse:')
        print('headers:', self.get_all_headers())


class ListExternalStoreResponse(LogResponse):
    """ The response of the list_logstore API from log.
    
    :type header: dict
    :param header: ListExternalStoreResponse HTTP response header

    :type resp: dict
    :param resp: the HTTP response body
    """

    def __init__(self, resp, header):
        LogResponse.__init__(self, header, resp)
        self.count = int(resp["count"])
        self.total_count = int(resp["total"])
        self.logstores = Util.convert_unicode_to_str(resp.get("externalstores", []))

    def get_logstores(self):
        """

        :return:
        """
        return self.logstores

    def get_count(self):
        return self.count

    def get_logstores_count(self):
        """

        :return:
        """
        return self.count

    def get_logstores_total(self):
        """

        :return:
        """
        return self.total_count

    def get_total(self):
        """

        :return:
        """
        return self.total_count

    def log_print(self):
        """

        :return:
        """
        print('ListExternalStoreResponse:')
        print('headers:', self.get_all_headers())
        print('logstores_count:', str(self.count))
        print('logstores_total:', str(self.total_count))
        print('logstores:', str(self.logstores))

    def merge(self, response):
        if not isinstance(response, ListExternalStoreResponse):
            raise ValueError("passed response is not a ListLogstoresResponse: " + str(type(response)))

        self.count += response.get_count()
        self.total = response.get_total()  # use the latest total count
        self.logstores.extend(response.get_logstores())

        # update body
        self.body = {
            'count': self.count,
            'total': self.total,
            'logstores': self.logstores
        }

        return self
